"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildShowroom = void 0;
const tslib_1 = require("tslib");
require('source-map-support').install();
// Do this as the first thing so that any code reading it knows the right env.
process.env.BABEL_ENV = 'production';
process.env.NODE_ENV = 'production';
const core_1 = require("@showroomjs/core");
const fs = (0, tslib_1.__importStar)(require("fs-extra"));
const path = (0, tslib_1.__importStar)(require("path"));
const perf_hooks_1 = require("perf_hooks");
const webpack_1 = (0, tslib_1.__importDefault)(require("webpack"));
const create_webpack_config_1 = require("../config/create-webpack-config");
const create_ssr_bundle_1 = require("../lib/create-ssr-bundle");
const generate_dts_1 = require("../lib/generate-dts");
const get_config_1 = require("../lib/get-config");
const log_to_stdout_1 = require("../lib/log-to-stdout");
const paths_1 = require("../lib/paths");
async function buildStaticSite(config, profile = false, measure = false) {
    const webpackConfig = (0, create_webpack_config_1.createClientWebpackConfig)('production', config, {
        outDir: config.outDir,
        profileWebpack: profile,
        measure,
    });
    const compiler = (0, webpack_1.default)(webpackConfig);
    try {
        await new Promise((fulfill, reject) => {
            compiler.run((err, stats) => {
                if (err || stats?.hasErrors()) {
                    if (err) {
                        console.error(err);
                    }
                    compiler.close(() => {
                        console.error('Fix the error and try again.');
                    });
                    reject(err);
                }
                compiler.close(() => {
                    fulfill();
                });
            });
        });
        const { manifest } = config.theme;
        if (manifest) {
            await fs.outputJSON((0, paths_1.resolveApp)(`${config.outDir}/manifest.json`), (0, core_1.omit)(manifest, ['baseIconPath']));
            if (manifest.baseIconPath) {
                await fs.copy((0, paths_1.resolveApp)(manifest.baseIconPath), (0, paths_1.resolveApp)(`${config.outDir}/_icons/${path.parse(manifest.baseIconPath).base}`));
            }
        }
    }
    catch (err) {
        console.error(err);
    }
}
async function prerenderSite(config, tmpDir) {
    const prerenderCodePath = `${tmpDir}/server/prerender.js`;
    const htmlPath = (0, paths_1.resolveApp)(`${config.outDir}/index.html`);
    const { ssr } = require(prerenderCodePath);
    const template = await fs.readFile(htmlPath, 'utf-8');
    const routes = await ssr.getRoutes();
    if (config.basePath !== '') {
        (0, log_to_stdout_1.logToStdout)(`Prerender with basePath: ${config.basePath}`);
    }
    let pageCount = 0;
    for (const route of routes) {
        if (route !== '') {
            pageCount++;
            await fs.outputFile((0, paths_1.resolveApp)(`${config.outDir}/${route}index.html`), await getHtml(`/${route}`));
        }
    }
    await fs.outputFile((0, paths_1.resolveApp)(`${config.outDir}/_offline.html`), await getHtml('/_offline'));
    await fs.outputFile(htmlPath, await getHtml('/'));
    async function getHtml(pathname) {
        const prerenderResult = await ssr.render({ pathname });
        const helmet = ssr.getHelmet();
        const finalHtml = template
            .replace('<!--SSR-style-->', `<style id="stitches">${ssr.getCssText()}</style>`)
            .replace('<!--SSR-helmet-->', `${helmet.title.toString()}${helmet.meta.toString()}${helmet.link.toString()}`)
            .replace('<!--SSR-target-->', prerenderResult.result);
        prerenderResult.cleanup();
        return finalHtml;
    }
    return pageCount + 1;
}
async function prerenderPreview(config, tmpDir) {
    const prerenderCodePath = `${tmpDir}/server/previewPrerender.js`;
    const htmlPath = (0, paths_1.resolveApp)(`${config.outDir}/_preview.html`);
    const { ssr } = require(prerenderCodePath);
    const template = await fs.readFile(htmlPath, 'utf-8');
    const routes = await ssr.getRoutes();
    let pageCount = 0;
    for (const route of routes) {
        if (route !== '') {
            pageCount++;
            await fs.outputFile((0, paths_1.resolveApp)(`${config.outDir}/_preview/${route}index.html`), await getHtml(`/${route}`));
        }
    }
    async function getHtml(pathname) {
        const prerenderResult = await ssr.render({ pathname });
        const helmet = ssr.getHelmet();
        const finalHtml = template
            .replace('<!--SSR-style-->', `<style id="stitches">${ssr.getCssText()}</style>`)
            .replace('<!--SSR-helmet-->', `${helmet.title.toString()}${helmet.meta.toString()}${helmet.link.toString()}`)
            .replace('<!--SSR-target-->', prerenderResult.result);
        prerenderResult.cleanup();
        return finalHtml;
    }
    return pageCount;
}
async function buildShowroom(userConfig, configFile, profile, measure) {
    const config = (0, get_config_1.getConfig)('production', configFile, userConfig);
    if (config.example.enableAdvancedEditor) {
        await (0, generate_dts_1.generateDts)(config, false);
    }
    const ssrDir = (0, paths_1.resolveShowroom)(`ssr-result-${Date.now() + perf_hooks_1.performance.now()}`);
    try {
        if (profile) {
            await buildStaticSite(config, profile, measure);
            await (0, create_ssr_bundle_1.createSSrBundle)(config, ssrDir, profile, measure);
        }
        else {
            await Promise.all([
                buildStaticSite(config, profile, measure),
                (0, create_ssr_bundle_1.createSSrBundle)(config, ssrDir, profile, measure),
            ]);
            (0, log_to_stdout_1.logToStdout)('Prerendering...');
            const [sitePageCount, previewPageCount] = await Promise.all([
                prerenderSite(config, ssrDir),
                prerenderPreview(config, ssrDir),
            ]);
            (0, log_to_stdout_1.logToStdout)((0, log_to_stdout_1.green)(`Prerendered ${sitePageCount + previewPageCount} pages.`));
            (0, log_to_stdout_1.logToStdout)(`Generated showroom at`);
            (0, log_to_stdout_1.logToStdout)(`  -  ${(0, log_to_stdout_1.green)((0, paths_1.resolveApp)(config.outDir))}`);
        }
    }
    finally {
        await fs.remove(ssrDir);
    }
}
exports.buildShowroom = buildShowroom;
//# sourceMappingURL=build-showroom.js.map